/**
 * Copyright (c) 2013, 2014, 2021 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 * Michael Golubev (Montages) - initial API and implementation
 * Florian Noyrit - Initial API and implementation
 * Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : 1.4 Merge papyrus extension templates into codegen.xtend
 * Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : L1.2 clean up
 */
package xpt.plugin;

import com.google.common.collect.Iterables;
import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.common.util.EList;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenDiagram;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenEditorGenerator;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenExpressionInterpreter;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenExpressionProviderContainer;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLanguage;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenLink;
import org.eclipse.papyrus.gmf.codegen.gmfgen.GenPlugin;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import plugin.Activator;
import xpt.CodeStyle;
import xpt.Common;
import xpt.editor.DocumentProvider;
import xpt.expressions.OCLExpressionFactory;
import xpt.expressions.getExpression;
import xpt.providers.ElementInitializers;

@Singleton
@SuppressWarnings("all")
public class ActivatorImpl {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private CodeStyle _codeStyle;
  
  @Inject
  private Activator xptActivator;
  
  @Inject
  private DocumentProvider xptDocProvider;
  
  @Inject
  private ElementInitializers xptElementInitializers;
  
  @Inject
  private getExpression xptExpr;
  
  @Inject
  private OCLExpressionFactory oclFactory;
  
  public CharSequence ActivatorImpl(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _copyright = this._common.copyright(it.getEditorGen());
    _builder.append(_copyright);
    _builder.newLineIfNotEmpty();
    _builder.append("package ");
    CharSequence _packageName = this.xptActivator.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedClassComment = this._common.generatedClassComment();
    _builder.append(_generatedClassComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public class ");
    CharSequence _className = this.xptActivator.className(it);
    _builder.append(_className);
    _builder.append(" extends org.eclipse.ui.plugin.AbstractUIPlugin {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _attrs = this.attrs(it);
    _builder.append(_attrs, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _constructor = this.constructor(it);
    _builder.append(_constructor, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _start = this.start(it);
    _builder.append(_start, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _stop = this.stop(it.getEditorGen());
    _builder.append(_stop, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _instance = this.getInstance(it);
    _builder.append(_instance, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _preferenceStore = this.getPreferenceStore(it);
    _builder.append(_preferenceStore, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _itemProvidersAdaptorFactory = this.getItemProvidersAdaptorFactory(it);
    _builder.append(_itemProvidersAdaptorFactory, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _itemImageDescriptor = this.getItemImageDescriptor(it);
    _builder.append(_itemImageDescriptor, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _bundleDescriptorImage = this.getBundleDescriptorImage(it);
    _builder.append(_bundleDescriptorImage, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _findImageDescriptor = this.findImageDescriptor(it);
    _builder.append(_findImageDescriptor, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _bundleImage = this.getBundleImage(it);
    _builder.append(_bundleImage, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _string = this.getString(it);
    _builder.append(_string, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _documentProviderGetter = this.documentProviderGetter(it.getEditorGen().getDiagram());
    _builder.append(_documentProviderGetter, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _linkConstraint = this.linkConstraint(it.getEditorGen().getDiagram());
    _builder.append(_linkConstraint, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _initializerGetter = this.initializerGetter(it.getEditorGen().getDiagram());
    _builder.append(_initializerGetter, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _initializerSetter = this.initializerSetter(it.getEditorGen().getDiagram());
    _builder.append(_initializerSetter, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _providersAccessMethods = this.providersAccessMethods(it);
    _builder.append(_providersAccessMethods, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _logError = this.logError(it);
    _builder.append(_logError, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _logInfo = this.logInfo(it);
    _builder.append(_logInfo, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    CharSequence _logError_1 = this.getLogError(it);
    _builder.append(_logError_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence attrs(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static final String ID = \"");
    String _iD = it.getID();
    _builder.append(_iD);
    _builder.append("\"; //$NON-NLS-1$");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.papyrus.infra.core.log.LogHelper myLogHelper;");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2);
    _builder.newLineIfNotEmpty();
    _builder.append("public static final org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint DIAGRAM_PREFERENCES_HINT =");
    _builder.append("new org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint(ID);");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_3 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_3);
    _builder.newLineIfNotEmpty();
    _builder.append("private static ");
    CharSequence _className = this.xptActivator.className(it);
    _builder.append(_className);
    _builder.append(" instance;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    CharSequence _generatedMemberComment_4 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_4);
    _builder.newLineIfNotEmpty();
    _builder.append("private org.eclipse.emf.common.notify.AdapterFactory adapterFactory;");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_5 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_5);
    _builder.newLineIfNotEmpty();
    _builder.append("private ");
    CharSequence _qualifiedClassName = this.xptDocProvider.qualifiedClassName(it.getEditorGen().getDiagram());
    _builder.append(_qualifiedClassName);
    _builder.append(" documentProvider;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      final Function1<GenLink, Boolean> _function = new Function1<GenLink, Boolean>() {
        @Override
        public Boolean apply(final GenLink l) {
          boolean _isSansDomain = l.isSansDomain();
          return Boolean.valueOf((!_isSansDomain));
        }
      };
      boolean _exists = IterableExtensions.<GenLink>exists(it.getEditorGen().getDiagram().getLinks(), _function);
      if (_exists) {
        CharSequence _generatedMemberComment_6 = this._common.generatedMemberComment();
        _builder.append(_generatedMemberComment_6);
        _builder.newLineIfNotEmpty();
        _builder.append("private ");
        String _linkCreationConstraintsQualifiedClassName = it.getEditorGen().getDiagram().getLinkCreationConstraintsQualifiedClassName();
        _builder.append(_linkCreationConstraintsQualifiedClassName);
        _builder.append(" linkConstraints;");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    CharSequence _generatedMemberComment_7 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_7);
    _builder.newLineIfNotEmpty();
    _builder.append("private ");
    CharSequence _qualifiedClassName_1 = this.xptElementInitializers.qualifiedClassName(it.getEditorGen().getDiagram());
    _builder.append(_qualifiedClassName_1);
    _builder.append(" initializers;");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      GenExpressionProviderContainer _expressionProviders = it.getEditorGen().getExpressionProviders();
      boolean _tripleNotEquals = (_expressionProviders != null);
      if (_tripleNotEquals) {
        {
          Iterable<GenExpressionInterpreter> _filter = Iterables.<GenExpressionInterpreter>filter(it.getEditorGen().getExpressionProviders().getProviders(), GenExpressionInterpreter.class);
          for(final GenExpressionInterpreter p : _filter) {
            CharSequence _generatedMemberComment_8 = this._common.generatedMemberComment();
            _builder.append(_generatedMemberComment_8);
            _builder.newLineIfNotEmpty();
            _builder.append("private ");
            CharSequence _expressionInterpriterQualifiedClassName = this.xptExpr.getExpressionInterpriterQualifiedClassName(p);
            _builder.append(_expressionInterpriterQualifiedClassName);
            _builder.append(" ");
            GenLanguage _language = p.getLanguage();
            _builder.append(_language);
            _builder.append("Factory;");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence constructor(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    CharSequence _className = this.xptActivator.className(it);
    _builder.append(_className);
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence start(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    CharSequence _overrideC = this._codeStyle.overrideC(it.getEditorGen().getDiagram());
    _builder.append(_overrideC);
    _builder.newLineIfNotEmpty();
    _builder.append("public void start(org.osgi.framework.BundleContext context) throws Exception {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("super.start(context);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("instance = this;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("myLogHelper = new org.eclipse.papyrus.infra.core.log.LogHelper(this);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.gmf.runtime.diagram.core.preferences.PreferencesHint.registerPreferenceStore(DIAGRAM_PREFERENCES_HINT, getPreferenceStore());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("adapterFactory = org.eclipse.papyrus.infra.gmfdiag.common.Activator.getInstance().getItemProvidersAdapterFactory();");
    _builder.newLine();
    _builder.append("\t");
    String _preferencesPackageName = it.getEditorGen().getDiagram().getPreferencesPackageName();
    _builder.append(_preferencesPackageName, "\t");
    _builder.append(".DiagramPreferenceInitializer diagramPreferenceInitializer = new ");
    String _preferencesPackageName_1 = it.getEditorGen().getDiagram().getPreferencesPackageName();
    _builder.append(_preferencesPackageName_1, "\t");
    _builder.append(".DiagramPreferenceInitializer();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("diagramPreferenceInitializer.initializeDefaultPreferences();");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence stop(final GenEditorGenerator it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    CharSequence _overrideC = this._codeStyle.overrideC(it.getDiagram());
    _builder.append(_overrideC);
    _builder.newLineIfNotEmpty();
    _builder.append("public void stop(org.osgi.framework.BundleContext context) throws Exception {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("adapterFactory = null;");
    _builder.newLine();
    {
      final Function1<GenLink, Boolean> _function = new Function1<GenLink, Boolean>() {
        @Override
        public Boolean apply(final GenLink l) {
          boolean _isSansDomain = l.isSansDomain();
          return Boolean.valueOf((!_isSansDomain));
        }
      };
      boolean _exists = IterableExtensions.<GenLink>exists(it.getDiagram().getLinks(), _function);
      if (_exists) {
        _builder.append("\t");
        _builder.append("linkConstraints = null;");
        _builder.newLine();
      }
    }
    _builder.append("\t");
    _builder.append("initializers = null;");
    _builder.newLine();
    {
      GenExpressionProviderContainer _expressionProviders = it.getExpressionProviders();
      boolean _tripleNotEquals = (_expressionProviders != null);
      if (_tripleNotEquals) {
        {
          Iterable<GenExpressionInterpreter> _filter = Iterables.<GenExpressionInterpreter>filter(it.getExpressionProviders().getProviders(), GenExpressionInterpreter.class);
          for(final GenExpressionInterpreter p : _filter) {
            _builder.append("\t");
            GenLanguage _language = p.getLanguage();
            _builder.append(_language, "\t");
            _builder.append("Factory = null;");
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    _builder.append("\t");
    _builder.append("instance = null;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("super.stop(context);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getInstance(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static ");
    CharSequence _className = this.xptActivator.className(it);
    _builder.append(_className);
    _builder.append(" getInstance() {");
    _builder.newLineIfNotEmpty();
    _builder.append("return instance;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence createAdapterFactory(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected org.eclipse.emf.edit.provider.ComposedAdapterFactory createAdapterFactory() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("java.util.ArrayList<org.eclipse.emf.common.notify.AdapterFactory> factories = new java.util.ArrayList<");
    String _diamondOp = this._codeStyle.diamondOp(it, "org.eclipse.emf.common.notify.AdapterFactory");
    _builder.append(_diamondOp, "\t");
    _builder.append(">();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("fillItemProviderFactories(factories);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return new org.eclipse.emf.edit.provider.ComposedAdapterFactory(factories);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence fillItemProviderFactories(final GenEditorGenerator it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("protected void fillItemProviderFactories(java.util.List<org.eclipse.emf.common.notify.AdapterFactory> factories) {");
    _builder.newLine();
    _builder.append("\t");
    CharSequence _populateItemProviderFactories = this.populateItemProviderFactories("factories", it);
    _builder.append(_populateItemProviderFactories, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("factories.add(new org.eclipse.emf.edit.provider.resource.ResourceItemProviderAdapterFactory());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("factories.add(new org.eclipse.emf.edit.provider.ReflectiveItemProviderAdapterFactory());");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence populateItemProviderFactories(final String factoryListVar, final GenEditorGenerator it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      EList<GenPackage> _allDomainGenPackages = it.getAllDomainGenPackages(true);
      for(final GenPackage genPackage : _allDomainGenPackages) {
        _builder.append(factoryListVar);
        _builder.append(".add(new ");
        String _qualifiedItemProviderAdapterFactoryClassName = genPackage.getQualifiedItemProviderAdapterFactoryClassName();
        _builder.append(_qualifiedItemProviderAdapterFactoryClassName);
        _builder.append("());");
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence getItemProvidersAdaptorFactory(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public org.eclipse.emf.common.notify.AdapterFactory getItemProvidersAdapterFactory() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return adapterFactory;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getItemImageDescriptor(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public org.eclipse.jface.resource.ImageDescriptor getItemImageDescriptor(Object item) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.emf.edit.provider.IItemLabelProvider labelProvider = (org.eclipse.emf.edit.provider.IItemLabelProvider) adapterFactory.adapt(item, org.eclipse.emf.edit.provider.IItemLabelProvider.class);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (labelProvider != null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return org.eclipse.emf.edit.ui.provider.ExtendedImageRegistry.getInstance().getImageDescriptor(labelProvider.getImage(item));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getBundleDescriptorImage(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment(
      ("Returns an image descriptor for the image file at the given\n" + "plug-in relative path."), 
      (("@param path\n" + "           the path\n") + "@return the image descriptor"));
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static org.eclipse.jface.resource.ImageDescriptor getBundledImageDescriptor(String path) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return org.eclipse.ui.plugin.AbstractUIPlugin.imageDescriptorFromPlugin(ID, path);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence findImageDescriptor(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment(
      (("Respects images residing in any plug-in. If path is relative,\n" + "then this bundle is looked up for the image, otherwise, for absolute\n") + "path, first segment is taken as id of plug-in with image"), 
      (("@param path\n" + "           the path to image, either absolute (with plug-in id as first segment), or relative for bundled images\n") + "@return the image descriptor"));
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static org.eclipse.jface.resource.ImageDescriptor findImageDescriptor(String path) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("final org.eclipse.core.runtime.IPath p = new org.eclipse.core.runtime.Path(path);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (p.isAbsolute() && p.segmentCount() > 1) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return org.eclipse.ui.plugin.AbstractUIPlugin.imageDescriptorFromPlugin(p.segment(0), p.removeFirstSegments(1).makeAbsolute().toString());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("} else {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return getBundledImageDescriptor(p.makeAbsolute().toString());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getBundleImage(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment(
      ("Returns an image for the image file at the given plugin relative path.\n" + "Client do not need to dispose this image. Images will be disposed automatically."), 
      (("@param path\n" + "           the path\n") + "@return image instance"));
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public org.eclipse.swt.graphics.Image getBundledImage(String path) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.swt.graphics.Image image = getImageRegistry().get(path);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (image == null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("getImageRegistry().put(path, getBundledImageDescriptor(path));");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("image = getImageRegistry().get(path);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return image;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getString(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment("Returns string from plug-in\'s resource bundle");
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static String getString(String key) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return org.eclipse.core.runtime.Platform.getResourceString(getInstance().getBundle(), \"%\" + key); ");
    CharSequence _nonNLS = this._common.nonNLS();
    _builder.append(_nonNLS, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence documentProviderGetter(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    CharSequence _qualifiedClassName = this.xptDocProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(" getDocumentProvider() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (documentProvider == null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("documentProvider = new ");
    CharSequence _qualifiedClassName_1 = this.xptDocProvider.qualifiedClassName(it);
    _builder.append(_qualifiedClassName_1, "\t\t");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return documentProvider;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence linkConstraint(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      final Function1<GenLink, Boolean> _function = new Function1<GenLink, Boolean>() {
        @Override
        public Boolean apply(final GenLink l) {
          boolean _isSansDomain = l.isSansDomain();
          return Boolean.valueOf((!_isSansDomain));
        }
      };
      boolean _exists = IterableExtensions.<GenLink>exists(it.getLinks(), _function);
      if (_exists) {
        CharSequence _linkConstraintsGetter = this.linkConstraintsGetter(it);
        _builder.append(_linkConstraintsGetter);
        _builder.newLineIfNotEmpty();
        CharSequence _linkConstraintsSetter = this.linkConstraintsSetter(it);
        _builder.append(_linkConstraintsSetter);
        _builder.newLineIfNotEmpty();
      }
    }
    return _builder;
  }
  
  public CharSequence linkConstraintsGetter(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    String _linkCreationConstraintsQualifiedClassName = it.getLinkCreationConstraintsQualifiedClassName();
    _builder.append(_linkCreationConstraintsQualifiedClassName);
    _builder.append(" getLinkConstraints() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return linkConstraints;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence linkConstraintsSetter(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void setLinkConstraints(");
    String _linkCreationConstraintsQualifiedClassName = it.getLinkCreationConstraintsQualifiedClassName();
    _builder.append(_linkCreationConstraintsQualifiedClassName);
    _builder.append(" lc) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.linkConstraints = lc;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence initializerGetter(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    CharSequence _qualifiedClassName = this.xptElementInitializers.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(" getElementInitializers() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return initializers;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence initializerSetter(final GenDiagram it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void setElementInitializers(");
    CharSequence _qualifiedClassName = this.xptElementInitializers.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(" i) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.initializers = i;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence providersAccessMethods(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    {
      GenExpressionProviderContainer _expressionProviders = it.getEditorGen().getExpressionProviders();
      boolean _tripleNotEquals = (_expressionProviders != null);
      if (_tripleNotEquals) {
        {
          Iterable<GenExpressionInterpreter> _filter = Iterables.<GenExpressionInterpreter>filter(it.getEditorGen().getExpressionProviders().getProviders(), GenExpressionInterpreter.class);
          for(final GenExpressionInterpreter p : _filter) {
            CharSequence _providerGetter = this.providerGetter(p);
            _builder.append(_providerGetter);
            _builder.newLineIfNotEmpty();
            CharSequence _providerSetter = this.providerSetter(p);
            _builder.append(_providerSetter);
            _builder.newLineIfNotEmpty();
          }
        }
      }
    }
    return _builder;
  }
  
  public CharSequence providerGetter(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public ");
    CharSequence _qualifiedClassName = this.oclFactory.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(" get");
    CharSequence _className = this.oclFactory.className(it);
    _builder.append(_className);
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("return ");
    GenLanguage _language = it.getLanguage();
    _builder.append(_language, "\t");
    _builder.append("Factory;");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence providerSetter(final GenExpressionInterpreter it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void set");
    CharSequence _className = this.oclFactory.className(it);
    _builder.append(_className);
    _builder.append("(");
    CharSequence _qualifiedClassName = this.oclFactory.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    _builder.append(" f) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("this.");
    GenLanguage _language = it.getLanguage();
    _builder.append(_language, "\t");
    _builder.append("Factory = f;");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence logError(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void logError(String error) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("getLogHelper().warn(error);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1);
    _builder.newLineIfNotEmpty();
    _builder.append("public void logError(String error, Throwable throwable) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("getLogHelper().error(error, throwable);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence logInfo(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public void logInfo(String message) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("getLogHelper().info(message);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1);
    _builder.newLineIfNotEmpty();
    _builder.append("public void logInfo(String message, Throwable throwable) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("getLogHelper().error(message, throwable);");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getLogError(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public org.eclipse.papyrus.infra.core.log.LogHelper getLogHelper() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return myLogHelper;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence preferenceHintAccess(final GenEditorGenerator it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.xptActivator.qualifiedClassName(it.getPlugin());
    _builder.append(_qualifiedClassName);
    _builder.append(".DIAGRAM_PREFERENCES_HINT");
    return _builder;
  }
  
  public CharSequence getPreferenceStore(final GenPlugin it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    CharSequence _overrideC = this._codeStyle.overrideC(it.getEditorGen().getDiagram());
    _builder.append(_overrideC);
    _builder.newLineIfNotEmpty();
    _builder.append("public org.eclipse.jface.preference.IPreferenceStore getPreferenceStore() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("org.eclipse.jface.preference.IPreferenceStore store=org.eclipse.papyrus.infra.gmfdiag.preferences.Activator.getDefault().getPreferenceStore();");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return store;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
